@@ -18,6 +18,7 @@ Key manager implementation for Barbican
 """
 import calendar
 import time
+import urllib
 
 from cryptography.hazmat import backends
 from cryptography.hazmat.primitives import serialization
@@ -32,11 +33,6 @@ from oslo_utils import excutils
 from castellan.common import exception
 from castellan.common.objects import key as key_base_class
 from castellan.common.objects import opaque_data as op_data
-from castellan.common.objects import passphrase
-from castellan.common.objects import private_key as pri_key
-from castellan.common.objects import public_key as pub_key
-from castellan.common.objects import symmetric_key as sym_key
-from castellan.common.objects import x_509
 from castellan.i18n import _
 from castellan.key_manager import key_manager
 
@@ -44,10 +40,9 @@ from castellan.key_manager import key_manager
 from barbicanclient import client as barbican_client_import
 from barbicanclient import exceptions as barbican_exceptions
 from oslo_utils import timeutils
-from six.moves import urllib
 
 
-barbican_opts = [
+_barbican_opts = [
     cfg.StrOpt('barbican_endpoint',
                help='Use this endpoint to connect to Barbican, for example: '
                     '"http://localhost:9311/"'),
@@ -69,7 +64,14 @@ barbican_opts = [
     cfg.BoolOpt('verify_ssl',
                 default=True,
                 help='Specifies if insecure TLS (https) requests. If False, '
-                     'the server\'s certificate will not be validated'),
+                     'the server\'s certificate will not be validated, if '
+                     'True, we can set the verify_ssl_path config meanwhile.'),
+    cfg.StrOpt('verify_ssl_path',
+               default=None,
+               help='A path to a bundle or CA certs to check against, or '
+                    'None for requests to attempt to locate and use '
+                    'certificates which verify_ssh is True. If verify_ssl '
+                    'is False, this is ignored.'),
     cfg.StrOpt('barbican_endpoint_type',
                default='public',
                choices=['public', 'internal', 'admin'],
@@ -78,7 +80,7 @@ barbican_opts = [
 
 ]
 
-BARBICAN_OPT_GROUP = 'barbican'
+_BARBICAN_OPT_GROUP = 'barbican'
 
 LOG = logging.getLogger(__name__)
 
@@ -86,20 +88,12 @@ LOG = logging.getLogger(__name__)
 class BarbicanKeyManager(key_manager.KeyManager):
     """Key Manager Interface that wraps the Barbican client API."""
 
-    _secret_type_dict = {
-        op_data.OpaqueData: 'opaque',
-        passphrase.Passphrase: 'passphrase',
-        pri_key.PrivateKey: 'private',
-        pub_key.PublicKey: 'public',
-        sym_key.SymmetricKey: 'symmetric',
-        x_509.X509: 'certificate'}
-
     def __init__(self, configuration):
         self._barbican_client = None
         self._base_url = None
         self.conf = configuration
-        self.conf.register_opts(barbican_opts, group=BARBICAN_OPT_GROUP)
-        loading.register_session_conf_options(self.conf, BARBICAN_OPT_GROUP)
+        self.conf.register_opts(_barbican_opts, group=_BARBICAN_OPT_GROUP)
+        loading.register_session_conf_options(self.conf, _BARBICAN_OPT_GROUP)
 
     def _get_barbican_client(self, context):
         """Creates a client to connect to the Barbican service.
@@ -122,8 +116,10 @@ class BarbicanKeyManager(key_manager.KeyManager):
 
         try:
             auth = self._get_keystone_auth(context)
-            sess = session.Session(auth=auth,
-                                   verify=self.conf.barbican.verify_ssl)
+            verify_ssl = self.conf.barbican.verify_ssl
+            verify_ssl_path = self.conf.barbican.verify_ssl_path
+            verify = verify_ssl and verify_ssl_path or verify_ssl
+            sess = session.Session(auth=auth, verify=verify)
 
             self._barbican_endpoint = self._get_barbican_endpoint(auth, sess)
             self._barbican_client = barbican_client_import.Client(
@@ -634,7 +630,7 @@ class BarbicanKeyManager(key_manager.KeyManager):
         except (barbican_exceptions.HTTPAuthError,
                 barbican_exceptions.HTTPClientError,
                 barbican_exceptions.HTTPServerError) as e:
-            LOG.error(_("Error listing objects: %s"), e)
+            LOG.error("Error listing objects: %s", e)
             raise exception.KeyManagerError(reason=e)
 
         for secret in secrets:
@@ -644,7 +640,10 @@ class BarbicanKeyManager(key_manager.KeyManager):
             except (barbican_exceptions.HTTPAuthError,
                     barbican_exceptions.HTTPClientError,
                     barbican_exceptions.HTTPServerError) as e:
-                LOG.warning(_("Error occurred while retrieving object "
-                              "metadata, not adding it to the list: %s"), e)
+                LOG.warning("Error occurred while retrieving object "
+                            "metadata, not adding it to the list: %s", e)
 
         return objects
+
+    def list_options_for_discovery(self):
+        return [(_BARBICAN_OPT_GROUP, _barbican_opts)]
