@@ -31,22 +31,18 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import timeutils
 import requests
-import six
 
 from castellan.common import exception
-from castellan.common.objects import opaque_data as op_data
-from castellan.common.objects import passphrase
 from castellan.common.objects import private_key as pri_key
 from castellan.common.objects import public_key as pub_key
 from castellan.common.objects import symmetric_key as sym_key
-from castellan.common.objects import x_509
 from castellan.i18n import _
 from castellan.key_manager import key_manager
 
-DEFAULT_VAULT_URL = "http://127.0.0.1:8200"
-DEFAULT_MOUNTPOINT = "secret"
+_DEFAULT_VAULT_URL = "http://127.0.0.1:8200"
+_DEFAULT_MOUNTPOINT = "secret"
 
-vault_opts = [
+_vault_opts = [
     cfg.StrOpt('root_token_id',
                help='root token for vault'),
     cfg.StrOpt('approle_role_id',
@@ -54,13 +50,13 @@ vault_opts = [
     cfg.StrOpt('approle_secret_id',
                help='AppRole secret_id for authentication with vault'),
     cfg.StrOpt('kv_mountpoint',
-               default=DEFAULT_MOUNTPOINT,
+               default=_DEFAULT_MOUNTPOINT,
                help='Mountpoint of KV store in Vault to use, for example: '
-                    '{}'.format(DEFAULT_MOUNTPOINT)),
+                    '{}'.format(_DEFAULT_MOUNTPOINT)),
     cfg.StrOpt('vault_url',
-               default=DEFAULT_VAULT_URL,
+               default=_DEFAULT_VAULT_URL,
                help='Use this endpoint to connect to Vault, for example: '
-                    '"%s"' % DEFAULT_VAULT_URL),
+                    '"%s"' % _DEFAULT_VAULT_URL),
     cfg.StrOpt('ssl_ca_crt_file',
                help='Absolute path to ca cert file'),
     cfg.BoolOpt('use_ssl',
@@ -68,7 +64,7 @@ vault_opts = [
                 help=_('SSL Enabled/Disabled')),
 ]
 
-VAULT_OPT_GROUP = 'vault'
+_VAULT_OPT_GROUP = 'vault'
 
 _EXCEPTIONS_BY_CODE = [
     requests.codes['internal_server_error'],
@@ -84,18 +80,10 @@ LOG = logging.getLogger(__name__)
 class VaultKeyManager(key_manager.KeyManager):
     """Key Manager Interface that wraps the Vault REST API."""
 
-    _secret_type_dict = {
-        op_data.OpaqueData: 'opaque',
-        passphrase.Passphrase: 'passphrase',
-        pri_key.PrivateKey: 'private',
-        pub_key.PublicKey: 'public',
-        sym_key.SymmetricKey: 'symmetric',
-        x_509.X509: 'certificate'}
-
     def __init__(self, configuration):
         self._conf = configuration
-        self._conf.register_opts(vault_opts, group=VAULT_OPT_GROUP)
-        loading.register_session_conf_options(self._conf, VAULT_OPT_GROUP)
+        self._conf.register_opts(_vault_opts, group=_VAULT_OPT_GROUP)
+        loading.register_session_conf_options(self._conf, _VAULT_OPT_GROUP)
         self._session = requests.Session()
         self._root_token_id = self._conf.vault.root_token_id
         self._approle_role_id = self._conf.vault.approle_role_id
@@ -174,21 +162,25 @@ class VaultKeyManager(key_manager.KeyManager):
                                           json=params,
                                           verify=self._verify_server)
             except requests.exceptions.Timeout as ex:
-                raise exception.KeyManagerError(six.text_type(ex))
+                raise exception.KeyManagerError(str(ex))
             except requests.exceptions.ConnectionError as ex:
-                raise exception.KeyManagerError(six.text_type(ex))
+                raise exception.KeyManagerError(str(ex))
             except Exception as ex:
-                raise exception.KeyManagerError(six.text_type(ex))
+                raise exception.KeyManagerError(str(ex))
 
             if resp.status_code in _EXCEPTIONS_BY_CODE:
                 raise exception.KeyManagerError(resp.reason)
             if resp.status_code == requests.codes['forbidden']:
                 raise exception.Forbidden()
 
-            resp = resp.json()
-            self._cached_approle_token_id = resp['auth']['client_token']
+            resp_data = resp.json()
+
+            if resp.status_code == requests.codes['bad_request']:
+                raise exception.KeyManagerError(', '.join(resp_data['errors']))
+
+            self._cached_approle_token_id = resp_data['auth']['client_token']
             self._approle_token_issue = token_issue_utc
-            self._approle_token_ttl = resp['auth']['lease_duration']
+            self._approle_token_ttl = resp_data['auth']['lease_duration']
             return {'X-Vault-Token': self._approle_token_id}
 
         return {}
@@ -200,11 +192,11 @@ class VaultKeyManager(key_manager.KeyManager):
         try:
             resp = method(resource, headers=headers, json=json, verify=verify)
         except requests.exceptions.Timeout as ex:
-            raise exception.KeyManagerError(six.text_type(ex))
+            raise exception.KeyManagerError(str(ex))
         except requests.exceptions.ConnectionError as ex:
-            raise exception.KeyManagerError(six.text_type(ex))
+            raise exception.KeyManagerError(str(ex))
         except Exception as ex:
-            raise exception.KeyManagerError(six.text_type(ex))
+            raise exception.KeyManagerError(str(ex))
 
         if resp.status_code in _EXCEPTIONS_BY_CODE:
             raise exception.KeyManagerError(resp.reason)
@@ -217,11 +209,6 @@ class VaultKeyManager(key_manager.KeyManager):
                         expiration=None, name=None):
         """Creates an asymmetric key pair."""
 
-        # Confirm context is provided, if not raise forbidden
-        if not context:
-            msg = _("User is not authorized to use key manager.")
-            raise exception.Forbidden(msg)
-
         if algorithm.lower() != 'rsa':
             raise NotImplementedError(
                 "VaultKeyManager only implements rsa keys"
@@ -293,11 +280,6 @@ class VaultKeyManager(key_manager.KeyManager):
     def create_key(self, context, algorithm, length, name=None, **kwargs):
         """Creates a symmetric key."""
 
-        # Confirm context is provided, if not raise forbidden
-        if not context:
-            msg = _("User is not authorized to use key manager.")
-            raise exception.Forbidden(msg)
-
         if length % 8:
             msg = _("Length must be multiple of 8.")
             raise ValueError(msg)
@@ -315,22 +297,12 @@ class VaultKeyManager(key_manager.KeyManager):
     def store(self, context, key_value, **kwargs):
         """Stores (i.e., registers) a key with the key manager."""
 
-        # Confirm context is provided, if not raise forbidden
-        if not context:
-            msg = _("User is not authorized to use key manager.")
-            raise exception.Forbidden(msg)
-
         key_id = uuid.uuid4().hex
         return self._store_key_value(key_id, key_value)
 
     def get(self, context, key_id, metadata_only=False):
         """Retrieves the key identified by the specified id."""
 
-        # Confirm context is provided, if not raise forbidden
-        if not context:
-            msg = _("User is not authorized to use key manager.")
-            raise exception.Forbidden(msg)
-
         if not key_id:
             raise exception.KeyManagerError('key identifier not provided')
 
@@ -371,11 +343,6 @@ class VaultKeyManager(key_manager.KeyManager):
     def delete(self, context, key_id):
         """Represents deleting the key."""
 
-        # Confirm context is provided, if not raise forbidden
-        if not context:
-            msg = _("User is not authorized to use key manager.")
-            raise exception.Forbidden(msg)
-
         if not key_id:
             raise exception.KeyManagerError('key identifier not provided')
 
@@ -388,11 +355,6 @@ class VaultKeyManager(key_manager.KeyManager):
     def list(self, context, object_type=None, metadata_only=False):
         """Lists the managed objects given the criteria."""
 
-        # Confirm context is provided, if not raise forbidden
-        if not context:
-            msg = _("User is not authorized to use key manager.")
-            raise exception.Forbidden(msg)
-
         if object_type and object_type not in self._secret_type_dict:
             msg = _("Invalid secret type: %s") % object_type
             raise exception.KeyManagerError(reason=msg)
@@ -412,7 +374,10 @@ class VaultKeyManager(key_manager.KeyManager):
                 if object_type is None or isinstance(obj, object_type):
                     objects.append(obj)
             except exception.ManagedObjectNotFoundError as e:
-                LOG.warning(_("Error occurred while retrieving object "
-                              "metadata, not adding it to the list: %s"), e)
+                LOG.warning("Error occurred while retrieving object "
+                            "metadata, not adding it to the list: %s", e)
                 pass
         return objects
+
+    def list_options_for_discovery(self):
+        return [(_VAULT_OPT_GROUP, _vault_opts)]
